/*

   fit function for combined interaction:
   Fit functions are implemented as external programms
   which will receive the following parameters:
   1. address of array (double) of x-values.
      (or shared memory ID in UNIX)
   2. address of array (double) of results.
      (or shared memory ID in UNIX)
   3. first array element
   4. last array element
   5. size of array in bytes (used for shmat() under UNIX !)
   6. 1st parameter (double)
   7. 2nd parameter (double)
      .
      .
      .
   26. last parameter (if used)
   On Amiga Computers the Address represents the
   real memory location in decimal format (%d)
   On UNIX Computers this parameter identifies
   a shared memory ID. See "examplefn.c"
   On Amiga Computers it is recommended to make
   the Fit function resident, since it will be
   called very often.

   This example contains the following parts:
   1. Commandline interface
   2. Help function (if no parameter was specified)
   3. (shared) Memory allocation
   4. a simple 'cos' function

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#ifdef UNIX
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

#define MAXPARM 30

#define PI180 0.0174533
#define PI4 12.566371
#define sqr(x) ((x)*(x))
#define EPS 1.0E-6
#define XTAL 5
#define POLY 0
#define sqm2 0.7071068
#define pi4  12.56637061435916
#define pi2  6.28318530717958
#define y0   0.282094791773878
#define zero 0
#define one  1

#define PM1(n) ((n)&1 ? (-1) : 1)
#define sqr(x) ((x)*(x))
#define SIGN(a,b) ((b)<0 ? -xfabs(a) : xfabs(a))
#define ZR(xr,xi,yr,yi) ((xr)*(yr)-(xi)*(yi))
#define ZI(xr,xi,yr,yi) ((xr)*(yi)+(xi)*(yr))
/* complex multiplication */
#define PM1(n) ((n)&1 ? (-1) : 1)

#define MTF 8
#define KMAX 4
#define MAXSPIN 4
#define MAXH 11
#define MAXYL 25
#define MAXW 36
#define MAXM 21

double *p;
double tchan, t0, spin, a0, tau, a22, a24, a42, a44,
       stop1 = 180.0,
       stop2 = 0.0,
       w0, eta, wb, beta_1, gamma_1,
       euler_amp, alpha_2, beta_2, gamma_2, damp,
       euler_frac, alpha_3, beta_3, gamma_3, fwhm;
       
double *shmx, *shmy;
int   shmidx, shmidy;
int   first, last;
int   max;
int   xtal_flag = POLY;
int   n_wkk;

typedef struct {
	  double pm[MAXM][18];
	  double w[MAXM][MAXW], sina[MAXM][MAXW], cosa[MAXM][MAXW];
	  int nm, ipm[MAXM][15], nwkk[MAXM];
	} wkkarea;

static wkkarea *q;

int xabs(x)
int x;
{
   if(x < 0) return(-x);
   return(x);
}

double xfabs(x)
double x;
{
double y;

   y = (double) fabs(x);
   return(y);
}

int round(a) 
double a;
{
return( (int)((a<0) ? a-0.5 : a+0.5));
}

double expp(x)
double x;
{  /* Avoids runtime error in exp-function */
double y;

   if (xfabs(x) < 88.0) y = exp(x);
   else if (x < 0) y = 0.0;
   else y = 1.0E20;
   return(y);
}   /* expp */

double pow10(x)
double x;
{
   return(expp(2.302585 * x));
}

void tqli(a, d, e, n, n2)
double *a, *d, *e;
int n,n2;
{
/*
QL algorith with implicit shifts, to determine the eigenvalues and
eigenvectors of a real, symmetric, tridiagonal matrix, or of a real,
symmetric matrix previously reduced by tred2. On input d[0..n-1] contains
the diagonal elements of the tridiagonal matrix. On output it returns the
eigenvalues. The vector e[0..n-1] inputs the subdiagonal elements of the
tridiagonal matrix, with e[0] arbritary. On output e is destroyed. On input
the matrix a[0..n-1,0..n-1] should be the identity matrix or a matrix that has
been reduced by tred2 if the eigenvectors are required. On output the k:th
column of a returns the normalized eigenvector corresponding to d[k].
Modified by Bengt Lindgren from a C-program in Numerical Recipies.
*/
int m,l,iter,i,k;
double s,q,p,g,f,dd,c,b;

      for (i=1; i<=n-1; i++) e[i-1]=e[i];
      e[n-1]=0.0;
      for (l=0; l<=n-1; l++) {
          iter=0;
          do {
	     for (m=l; m<=n-2; m++) {
                 dd=xfabs(d[m])+xfabs(d[m+1]);
                 if ( (xfabs(e[m])+dd) == dd) break;
		 }      /* if not break m=n-2+1 here */
             if (m != l) {
                if (iter++ == 30) {
		   printf("Too many iterations in TQLI\n");
                   return;
                   }
                g=(d[l+1]-d[l])/(2.0*e[l]);
                q=sqrt(g*g+1.0);
                g=d[m]-d[l]+e[l]/(g+SIGN(q,g));
                s=c=1.0;
                p=0.0;
                for (i=m-1; i>=l; i--) {
                    f=s*e[i];
                    b=c*e[i];
                    if (xfabs(f) >= xfabs(g)) {
                       c=g/f;
                       q=sqrt(c*c+1.0);
                       e[i+1]=f*q;
                       c*= (s=1.0/q);
                    } else {
                       s=f/g;
                       q=sqrt(s*s+1.0);
                       e[i+1]=g*q;
                       s*= (c=1.0/q);
                    }
                    g=d[i+1]-p;
                    q=(d[i]-g)*s+2.0*c*b;
                    p=s*q;
                    d[i+1]=g+p;
                    g=c*q-b;
                    for (k=0; k<=n-1; k++) {
                        f=a[k*n2+i+1];
                        a[k*n2+i+1]=s*a[k*n2+i]+c*f;
                        a[k*n2+i]=c*a[k*n2+i]-s*f;
                        }
                    }
                d[l]=d[l]-p;
                e[l]=g;
                e[m]=0.0;
                }
          } while (m != l);
       }
} /*tqli*/

void tred2(a, d, e, n, n2) 
double *a, *d, *e;
int n, n2;
{
/*
Householder reduction of a real symmetric matrix a[0..n-1,0..n-1]. On output a
is replaced by the orthogonal matrix Q effecting the transformation.
d[0..n-1] returns the diagonal elements of the tridiagonal matrix, and
e[0..n-1] the off-diagonal elements, with e[0]=0.
n2 is the size of the second dimension of a in the calling program.
Modified by Bengt Lindgren from a C-program in Numerical Recipies.
*/
int l,k,j,i;
double scale,hh,h,g,f;

   for (i=n-1; i>=1; i--) {
      l=i-1;
      h=scale=0.0;
      if (l > 0) {
         for (k=0; k<=l; k++) scale+=xfabs(a[i*n2+k]);
         if (xfabs(scale) < EPS) e[i]=a[i*n2+l];
         else {
            for (k=0; k<=l; k++) {
               a[i*n2+k]/=scale;
               h+=a[i*n2+k]*a[i*n2+k];
            }
            f=a[i*n2+l];
            g= f>0 ? -sqrt(h): sqrt(h);
            e[i]=scale*g;
            h-=f*g;
            a[i*n2+l]=f-g;
            f=0.0;
            for (j=0; j<=l; j++) {
               a[j*n2+i]=a[i*n2+j]/h;
               g=0.0;
               for (k=0; k<=j; k++) g+=a[j*n2+k]*a[i*n2+k];
               for (k=j+1; k<=l; k++) g+=a[k*n2+j]*a[i*n2+k];
               e[j]=g/h;
               f+=e[j]*a[i*n2+j];
            }
            hh=f/(h+h);
            for (j=0; j<=l; j++) {
               f=a[i*n2+j];
               e[j]=g=e[j]-hh*f;
               for (k=0; k<=j; k++) a[j*n2+k]-=(f*e[k]+g*a[i*n2+k]);
            }
         }
      } else e[i]=a[i*n2+l];
      d[i]=h;
   }
   d[0]=0.0;
   e[0]=0.0;
   for (i=0; i<=n-1; i++) {
      l=i-1;
      if (d[i]) {
         for (j=0; j<=l; j++) {
            g=0.0;
            for (k=0; k<=l; k++) g+=a[i*n2+k]*a[k*n2+j];
            for (k=0; k<=l; k++) a[k*n2+j]-=g*a[k*n2+i];
         }
      }
      d[i]=a[i*n2+i];
      a[i*n2+i]=1.0;
      for (j=0; j<=l; j++) a[j*n2+i]=a[i*n2+j]=0.0;
   }
} /*tred2*/


void cymqr(a, b, d, n, n2)
double *a, *b, *d;
int n,n2;
{
/*
Matrix diagonalization of a complex Hermitian matrix a+ib of dimension n x n
by the Householder and QL methods.
n2 is the second dimension of a[][n2] and b[][n2] in the calling program.
On output d is the n eigenvalues, the k:th column of a,b is the normalized
eigenvector corresponding to d[k].
The phasefactor exp(ix) is arbitrary.
*/
double *wrk, *c, *e;
int m,i,j,jk,k,kk,kkk;

	m=n+n;
     c= (double*) malloc(m*m*sizeof(double));
     e= (double*) malloc(m*sizeof(double));
     wrk= (double*) malloc(m*sizeof(double));
     if (wrk == NULL || e == NULL || c == NULL) {
	printf(" malloc failed in cymqr\n");
	exit(0);
	}
	/* form the real matrix | a -b |
					    | b  a | which is symmetric if (a,b) Hermitian */
	for (i=0; i<n; i++)
		for (j=0; j<n; j++) {
			c[m*i+j]=a[i*n2+j];
			c[m*i+n+j]=-b[i*n2+j];
			c[m*(n+i)+n+j]=a[i*n2+j];
			c[m*(n+i)+j]=b[i*n2+j];
			}

     tred2(c,e,wrk,m,m);
     tqli (c,e,wrk,m,m);

	/* extract eigenvalues and vectors from the double set */
	 k=0;
	 d[k]=e[k];
	 for (j=0; j<n; j++) {
		a[n2*j]=c[m*j];
		b[n2*j]=c[m*(j+n)];
		}
      for (i=1; i<m; i++) {
	      /* test if "i" is already present */
	   kkk=1;
	   for (jk=i-1; jk>=0; jk--) {
		kk=0;
		for (j=0; j<n; j++) {
		   if(xfabs(c[m*j+i])-xfabs(c[m*(j+n)+jk]) >EPS) kk=1;
		   if(xfabs(c[m*(j+n)+i])-xfabs(c[m*j+jk]) >EPS) kk=1;
		   if(kk)break;  /* this vector is different */
		   }
		kkk=kkk && kk;
		if(! kkk)break;  /* an equivalent vector found */
		}
		if(kkk) {  /* all old vectors (jk) were different */
			d[++k]=e[i];
			for (j=0; j<n; j++) {
				a[j*n2+k]=c[m*j+i];
				b[j*n2+k]=c[m*(j+n)+i];
				}
			}
		}
		if(k!=n-1) {
			printf("Error in cymqr: Extraction of eigenvectors failed.\n");
			for(i=0; i<m; i++){
				printf("%3d %lf\n",i,e[i]);
				for(j=0; j<n; j++) printf("%8.4f",c[m*j+i]);
				printf("\n");
				for(j=n; j<m; j++) printf("%8.4f",c[m*j+i]);
				printf("\n");
				}
			printf("Error in cymqr: Extraction of eigenvectors failed.\n");
			exit(0);
			}

     free(wrk);
	free(e);
	free(c);
} /*cymqr*/

void symqr(a, d, n, n2)
double *a, *d;
int n,n2;
{
/*
Matrix diagonalization of a real symmetric matrix a of dimension n x n
by the Householder and QL methods.
n2 is the second dimension of a[][n2] in the calling program.
On output d is the n eigenvalues, the k:th column of a the normalized
eigenvector corresponding to d[k].
*/
double *wrk;

   wrk= (double*) malloc(n*sizeof(double));
   if (wrk == NULL) {
      printf(" malloc failed in symqr\n");
      exit(0);
   }
   tred2(a,d,wrk,n,n2);
   tqli (a,d,wrk,n,n2);
   free(wrk);
} /*symqr*/

void ylm(lmax, z, x, cosphi, sinphi, ylr, yli)
int lmax;
double z, x, cosphi, sinphi, *ylr, *yli;
{
/*    complex spherical harmonic subroutine Edmonds phase convention
      modified from real spherical harmonic routine ylm1 by BL -sept88
      dec 76, linear storage, index=l*(l+1)+m   l=0..lmax, m=-l..l */

/*  (-1)**n  */
 int kbl,kbm,kbh,kbp,mfac2,lfac,mfac,m,m1,l,lm,lp;
 double sinmp,cosmp,costp,pmm,plm,plm1,plm0,fac,fac1,yl1;

   plm0 = 0.0;
	ylr[0]=y0;
	yli[0]=zero;
	if (lmax<=0) return;
	sinmp=zero;
	cosmp=one;
	pmm=one;
	fac=one;
	kbl=0;
	mfac2=0;
	mfac=1;
/*     p(m,m) by recursion formula*/
	for(m=0; m<=lmax; m++){
      kbh=kbl+m+m;
      if (m==0) fac1=fac/pi4;
	 else {
      	fac=fac/(mfac*mfac2);
      	pmm=-pmm*mfac*x;
      	costp=cosmp*cosphi-sinmp*sinphi;
      	sinmp=sinmp*cosphi+cosmp*sinphi;
      	cosmp=costp;
      	fac1=fac/pi2;
      	mfac=mfac+2;
      	yl1=pmm* sqrt(fac1*mfac);
		ylr[kbh]=yl1*cosmp*sqm2;
		yli[kbh]=yl1*sinmp*sqm2;
		ylr[kbl]=ylr[kbh]*PM1(m);
		yli[kbl]=-yli[kbh]*PM1(m);
      	if (m==lmax) return;
	 }
      kbm=kbh+2;
      lfac=mfac;
      plm=pmm;
      mfac2=mfac2+2;
      m1=m+1;
/*     recursion for p(l,m), l = m+1 to lmax.*/
	 for(l=m1; l<=lmax; l++){
      	kbp=kbm+m+m;
      	lm=l-m;
      	lp=l+m;
      	plm1=plm*z*lfac;
      	lfac=lfac+2;
	     if (l != m1) plm1=plm1-(lp-1)*plm0;
      	plm1=plm1/lm;
      	if (m==0) {
      		yl1= sqrt(lfac*fac1)*plm1;
			ylr[kbp]=yl1*cosmp;
			yli[kbp]=zero;
      	} else {
      		fac1=lm*fac1/lp;
    		     yl1= sqrt(lfac*fac1)*plm1;
			ylr[kbp]=yl1*cosmp*sqm2;
			yli[kbp]=yl1*sinmp*sqm2;
			ylr[kbm]=ylr[kbp]*PM1(m);
			yli[kbm]=-yli[kbp]*PM1(m);
		}
      	plm0=plm;
      	plm=plm1;
          kbm=kbm+l+l+2;
	 }
      kbl=kbh+1;
	}
     return;
}  /* end ylm */

double trij(x1, x2, x3, x4, x5, x6)
double x1, x2, x3, x4, x5, x6;
{
/*   calculates 3J-symbols
     translated from fortran by Bengt Lindgren
*/
 /*sqrt(n!) with mantissa in fact[0..89] and power of 10 in nfact[0..89]*/
static double fact[90] = { 1.0000000000E+00,
 1.0000000000E+00, 1.4142135624E+00, 2.4494897428E+00, 4.8989794856E+00,
 1.0954451150E+00, 2.6832815730E+00, 7.0992957397E+00, 2.0079840637E+00,
 6.0239521910E+00, 1.9049409440E+00, 6.3179743589E+00, 2.1886105181E+00,
 7.8911474451E+00, 2.9525970128E+00, 1.1435359059E+00, 4.5741436235E+00,
 1.8859677306E+00, 8.0014834286E+00, 3.4877657663E+00, 1.5597762686E+00,
 7.1477928182E+00, 3.3526120082E+00, 1.6078562355E+00, 7.8768547132E+00,
 3.9384273566E+00, 2.0082117944E+00, 1.0434974581E+00, 5.5216695357E+00,
 2.9735100460E+00, 1.6286585272E+00, 9.0679869068E+00, 5.1296280268E+00,
 2.9467469553E+00, 1.7182339743E+00, 1.0165209278E+00, 6.0991255668E+00,
 3.7099532465E+00, 2.2869687743E+00, 1.4282115418E+00, 9.0328029052E+00,
 5.7838159215E+00, 3.7483411234E+00, 2.4579516485E+00, 1.6304206742E+00,
 1.0937194378E+00, 7.4179661362E+00, 5.0855013668E+00, 3.5233386997E+00,
 2.4663370898E+00, 1.7439636809E+00, 1.2454391809E+00, 8.9809896543E+00,
 6.5382591598E+00, 4.8046196243E+00, 3.5632012789E+00, 2.6664556771E+00,
 2.0131298891E+00, 1.5331540468E+00, 1.1776379688E+00, 9.1219444817E+00,
 7.1244663932E+00, 5.6098104478E+00, 4.4526490041E+00, 3.5621192033E+00,
 2.8718723147E+00, 2.3331200978E+00, 1.9097411060E+00, 1.5748128595E+00,
 1.3081378078E+00, 1.0944666130E+00, 9.2221396030E+00, 7.8252449404E+00,
 6.6858922079E+00, 5.7514219473E+00, 4.9808775142E+00, 4.3422283469E+00,
 3.8102899106E+00, 3.3651569322E+00, 2.9910169058E+00, 2.6752468493E+00,
 2.4077221644E+00, 2.1802851504E+00, 1.9863343046E+00, 1.8205054613E+00,
 1.6784231035E+00, 1.5565055536E+00, 1.4518117297E+00, 1.3619201234E+00,
 1.2848328748E+00};
static int nfact[90] = {  0,
  0,  0,  0,  0,  1,  1,  1,  2,  2,  3,
  3,  4,  4,  5,  6,  6,  7,  7,  8,  9,
  9, 10, 11, 11, 12, 13, 14, 14, 15, 16,
 16, 17, 18, 19, 20, 20, 21, 22, 23, 23,
 24, 25, 26, 27, 28, 28, 29, 30, 31, 32,
 33, 33, 34, 35, 36, 37, 38, 39, 40, 40,
 41, 42, 43, 44, 45, 46, 47, 48, 49, 50,
 50, 51, 52, 53, 54, 55, 56, 57, 58, 59,
 60, 61, 62, 63, 64, 65, 66, 67, 68};
static double eps = 10.e-4;
int na,nb,nc,nd,ne,nf,l8,l10,l3,l1,l2,l9,l11,l6,l7,l4,l5,l0,
     kmin,kmax,k,nco1,nden;
double trijd,co1,sgn,den;

      trijd=0.0;
      na=round(2*x1);
      nb=round(2*x2);
      nc=round(2*x3);
      nd=round(2*x4);
      ne=round(2*x5);
      nf=round(2*x6);
      if (xfabs(x4+x5+x6) > eps) return(0);  /*invalid input*/
      l8=round(x1+x4);
      l10=round(x3+x6);
      l3=round(x2+x5);
      if (xfabs(x2+x5-l3) > eps) return(0);
      if (xfabs(x1+x4-l8) > eps) return(0);
      if (xfabs(x3+x6-l10) > eps) return(0);
      l0=l3+l8+l10;
      if (l0 > 99) {
         printf("msg from trij: division by zero\n");
         return(0);
         }
      if ((xabs(nd)>na) || (xabs(ne)>nb) || (xabs(nf)>nc) ) return(0);
      l1=l0-nc;
      l2=l8-nd;
      l9=l3-ne;
      l11=l10-nf;
      l6=l0-nb;
      l7=l0-na;
      l4=l6-l2;
      l5=l7-l3;
      if (-l4 > -l5) kmin= -l4; else kmin= -l5;
      if (kmin<0) kmin=0;
      if (l2<l3) kmax=l2; else kmax=l3;
      if (kmax>l1) kmax=l1;
      if (kmax < kmin) return(0);
      co1=fact[l1]*fact[l6]*fact[l7]*fact[l8]*fact[l2]*fact[l3]*
           fact[l9]*fact[l10]*fact[l11]/fact[l0+1];
      nco1=nfact[l1]+nfact[l6]+nfact[l7]+nfact[l8]+nfact[l2]+nfact[l3]+
            nfact[l9]+nfact[l10]+nfact[l11]-nfact[l0+1];
      sgn=1.0;
   for (k=kmin; k<=kmax; k++) {
      den=fact[k]*fact[l1-k]*fact[l2-k]*fact[l3-k]*fact[l4+k]*fact[l5+k];
      nden=nfact[k]+nfact[l1-k]+nfact[l2-k]+nfact[l3-k]+nfact[l4+k]+nfact[l5+k];
      trijd+=sgn*co1/(den*den)*pow10((double)(nco1-2*nden));
      sgn= -sgn;
   }
   if ( ((l6-l10+kmin) % 2) != 0 ) trijd= -trijd;
	 return (trijd);
}

double f_xtal(xx)
double xx;
{
double  f, t, ff, d, a, sigma,t1,t2,fff,x,z,tt;
int i;

   tt = xx + t0;
   if(tau>0.0 && fwhm >= tchan) {  /* do convolution to gaussian prompt */
      sigma=fwhm/2.355;
      t1=tt-fwhm;
      t2=tt+fwhm;
   } else t1=t2=tt;
   f = a0;
   if(euler_amp!=0.0) {
      a=euler_amp;
      d=damp;
      t=t1;
      fff=z=0;
      while(t<=t2) {
         if(xfabs(tau) > EPS && fwhm >= tchan) {
            if(t >= 0.0) {
               x=expp(-0.5*sqr((tt-t)/sigma)-t/tau);
               if(t==t1) x*=0.5;  /* trapezoid rule factor */
            } else x=0;
            z+=x;
         } else x=z=1; /* no convolution */
         if(d < 0) a*=expp(d*t);  /* lorentz damping */
         if(x > 0.0) for(i=0; i<(int)q->nwkk[n_wkk]; i++) {
            ff=q->cosa[n_wkk][i]*cos(q->w[n_wkk][i]*t)+
               q->sina[n_wkk][i]*sin(q->w[n_wkk][i]);
            if(d>0) ff*=expp(-0.5*sqr(d*q->w[n_wkk][i]*t));  /* gaussian damping */
            fff+=a*ff*x;
         }
         t += tchan;
      }  /* while t */
      f+=fff/z;
   } /* for set */

   return(f);
}   /* fnc */


double fn(xx) /* (2/3)*(W1-W2) */
double xx;
{

   if(xtal_flag == POLY) {
      a24 = a42 = 0.0;    /* A24, A42 */
      gamma_1 = 0.0;  /* gamma */
      euler_frac = 0.0;  /* fraction of extra Euler angles */
   }
   return(f_xtal(xx));   /* calculate function  */
}   /* fnc */


int prewkk()    /* returns index of memo */
{
double a,b,c;
int i,n,ntf,j,k;
double ctf1[MTF][4],ctf2[MTF][4],wtf[MTF];


   q->nm=(q->nm+1) % MAXM;
   n=q->nm;

   if(xtal_flag==XTAL) {   /* single xtal */
      ntf=0;
      for(i=1; i<=2; i++) if(i==1 || p[10+5*i] !=0) {
         k=ntf;
         if(i==1) {
            wtf[ntf]=1;
            for(j=2; j<=2; j++) wtf[ntf]-=p[10+5*j];
         } else wtf[ntf]=p[10+5*i];
         a=p[10+5*i+1];
         b=p[10+5*i+2];
         c=p[10+5*i+3];

         rotate(90.0,0.0,a,b,c,ctf1,ntf);
         rotate(90.0,stop1,a,b,c,ctf2,ntf);
         ntf++;
         if(stop2 != 0) {
            wtf[ntf]=-wtf[k];
            for(j=0; j<4; j++) ctf1[ntf][j]=ctf1[ntf-1][j];
            rotate(90.0,stop2,a,b,c,ctf2,ntf);
            ntf++;
            wtf[ntf]=wtf[k];
            rotate(90.0,90.0,a,b,c,ctf1,ntf);
            rotate(90.0,stop2,a,b,c,ctf2,ntf);
            ntf++;
            wtf[ntf]=-wtf[k];
            for(j=0; j<4; j++) ctf1[ntf][j]=ctf1[ntf-1][j];
            rotate(90.0,stop1,a,b,c,ctf2,ntf);
            ntf++;
            for(j=k; j<ntf; j++) wtf[j] /= 2;
         }  /* if stop2 */
      }
   } else {   /* poly xtal */
      ctf2[0][0]=cos(stop1*PI180);  ctf2[0][1]=sin(stop1*PI180);
      ctf2[0][2]=1;          	    ctf2[0][3]=0;
      ctf2[1][0]=cos(stop2*PI180); ctf2[1][1]=sin(stop2*PI180);
      ctf2[1][2]=1;                ctf2[1][3]=0;
      if(stop2 != 0) ntf=-1; else ntf=0;  /* used as random flag in wkk */
   }

/* test print of rotated cos
	for(j=0; j<ntf; j++) printf("cost1,sint1,cosf1,sinf1: %lf %lf %lf %lf\n",
					ctf1[j][0],ctf1[j][1],ctf1[j][2],ctf1[j][3]);
	for(j=0; j<ntf; j++) printf("cost2,sint2,cosf2,sinf2: %lf %lf %lf %lf\n",
					ctf2[j][0],ctf2[j][1],ctf2[j][2],ctf2[j][3]);
*/

   q->nwkk[n] = wkk(spin, (p+5), w0,eta,wb,beta_1,gamma_1,
               ntf, wtf, ctf1, ctf2,q->w,q->cosa,q->sina,n);

   return(n);
}  /* end of prewkk */


rotate(t,f,a,b,g,ctf,ntf)
double t,f,a,b,g,ctf[MTF][4];
int ntf;
{ /* rotate theta,phi (t,f) the Euler angles (a,b,g): Rznew(g) Rynew(b) Rz(a) */
double x,y,z,xx,yy,zz, ca,sa,cb,sb,cg,sg;

   t*=PI180; f*=PI180; a*=PI180; b*=PI180; g*=PI180;
   xx=sin(t);
   yy=xx*sin(f);
   xx*=cos(f);
   zz=cos(t);
   ca=cos(a); sa=sin(a);
   cb=cos(b); sb=sin(b);
   cg=cos(g); sg=sin(g);
   x=xx*( ca*cb*cg-sa*sg) + yy*( sa*cb*cg+ca*sg) - zz*sb*cg;
   y=xx*(-ca*cb*sg-sa*cg) + yy*(-sa*cb*sg+ca*cg) + zz*sb*sg;
   z=xx*ca*sb             + yy*sa*sb             + zz*cb   ;
   ctf[ntf][0]=z;
   if(xfabs(z) < 1) {
      ctf[ntf][1]=sqrt(1.0-z*z);
      ctf[ntf][2]=x/ctf[ntf][1];
      ctf[ntf][3]=y/ctf[ntf][1];
   } else {
      ctf[ntf][1]=0;  /* theta 0 - 180 */
      ctf[ntf][2]=1;
      ctf[ntf][3]=0;
   }
   return(0);
}  /* end rotate */


wkk(spin,akk,w0,eta,wb,beta,gamma,ntf,wtf,ctf1,ctf2,w,cosa,sina,n_)
double spin,akk[2][2],w0,eta,wb,beta,gamma;
int ntf;
double *wtf,ctf1[MTF][4],ctf2[MTF][4],
       w[MAXM][MAXW],cosa[MAXM][MAXW],sina[MAXM][MAXW];
int n_;
{
/*  calc. of wkk for noncoaxial combined electric and magnetic interaction.
    returns number of frequency components w including w=0;
    if error wkk returns 0
*/
double ev[MAXH],hr[MAXH][MAXH],hi[MAXH][MAXH],ar[MAXH][MAXH],ai[MAXH][MAXH];
double y1r[MTF][MAXYL], y1i[MTF][MAXYL], y2r[MTF][MAXYL], y2i[MTF][MAXYL];
double trija[MAXH][KMAX+KMAX+1], trijb[MAXH][KMAX+KMAX+1];
double yyr,yyi,tr,ti,pr,pi,tmp,we;
int k,k1,k2,n,nspin,i,j,np,n1,n2,ma,mb,map,mbp,k1k1,k2k2,kmx;

	if((spin<=0) || (spin>MAXSPIN) ) return 0;
     nspin=2*spin+1;
	if(xfabs(akk[1][1])<EPS) kmx=2; else kmx=KMAX;

	 for (i=0; i<nspin; i++)  for (j=0; j<nspin; j++) {
	 	hr[i][j]=0;
	 	hi[i][j]=0;
		ar[i][j]=0;
		ai[i][j]=0;
	 }
/* this part makes we=w0/3 for integer spin and we=w0/6 for half-integer spin*/
      if(xfabs(spin-(int)spin ) > 0.1) we=w0/6;
	 else we=w0/3;
/*	calc. of matrix elements; i=0..nspin-1  <==> m= +I..-I */
	for (i=0; i<nspin; i++)
	   hr[i][i]=we*(3*(spin-i)*(spin-i)-spin*(spin+1))
			 +wb*(spin-i)*cos(beta*PI180);
	for (i=0; i<nspin-1; i++) {
	   tmp=wb*0.5*sin(beta*PI180)*sqrt(spin*(spin+1)-(spin-i)*(spin-i-1));
	   hr[i][i+1]=tmp*cos(gamma*PI180);
	   hi[i][i+1]=tmp*sin(gamma*PI180); /* opposite to Bostrom et al. !! */
	   hr[i+1][i]=hr[i][i+1];
	   hi[i+1][i]=-hi[i][i+1];
	   }
	for (i=0; i<nspin-2; i++) {
	   hr[i][i+2]=we*eta*0.5*sqrt((2*spin-i-1)*(2*spin-i)*(i+1)*(i+2));
	   hr[i+2][i]=hr[i][i+2];
	   }

	if(xfabs(sin(gamma*PI180)) > EPS) cymqr(hr,hi,ev,nspin,MAXH);
	else      symqr(hr,ev,nspin,MAXH);  /* only real part needed */


	if(ntf>0) for(n=0; n<ntf; n++) {
	  ylm(kmx,ctf1[n][0],ctf1[n][1],ctf1[n][2],ctf1[n][3],y1r[n],y1i[n]);
	  ylm(kmx,ctf2[n][0],ctf2[n][1],ctf2[n][2],ctf2[n][3],y2r[n],y2i[n]);
	} else for(n=0; n<=1; n++)
	  ylm(kmx,ctf2[n][0],ctf2[n][1],ctf2[n][2],ctf2[n][3],y2r[n],y2i[n]);
		/* ylm for ntf=0 used to calculate Plm(cos theta) */


  for (k1=2; k1<=kmx; k1+=2)
   for (k2=2; k2<=kmx; k2+=2) if(akk[k1/2-1][k2/2-1] != 0.0) {
	k1k1=k1*k1;
	k2k2=k2*k2;

/*	calculate trij symbols */
     for (n=-k1; n<=k1; n++)
		for (i=0; i<nspin; i++) {                        /* M= +I..-I */
			tmp=-(spin-i);                              /* -M        */
			if(xfabs(-n-tmp)<=spin+EPS)
			   trija[i][n+k1]=trij(spin,spin,(double)k1,
							   -n-tmp,tmp,(double)n);
			else trija[i][n+k1]=0;                      /* 3J:(-N+M,-M,N)*/
		}
if(ntf>0) {  /* full case with single xtal */

	we=PI4*akk[k1/2-1][k2/2-1];
	for (n=-k2; n<=k2; n++)
		for (i=0; i<nspin; i++) {
			tmp=-(spin-i);
			if(xfabs(-n-tmp)<=spin+EPS)
			   trijb[i][n+k2]=trij(spin,spin,(double)k2,
							   -n-tmp,tmp,(double)n);
			else trijb[i][n+k2]=0;
		}  /* else (ntf=0) trijb=trija */

/*   start summation */
	for (n1=0; n1<=k1+k1; n1++)              /* -K1..+K1 */
	  for (n2=0; n2<=k2+k2; n2++) {          /* -K2..+K2 */
	    yyr=yyi=0;
	    for(n=0; n<ntf; n++) {
		 yyr+=wtf[n]*ZR(y1r[n][k1k1+n1],-y1i[n][k1k1+n1],
					 y2r[n][k2k2+n2], y2i[n][k2k2+n2]);
		 yyi+=wtf[n]*ZI(y1r[n][k1k1+n1],-y1i[n][k1k1+n1],
		                y2r[n][k2k2+n2], y2i[n][k2k2+n2]);
		}
		yyr*=we;
		yyi*=we;
		if(xfabs(yyr)>EPS || xfabs(yyi)>EPS) {
		  for (ma=0; ma<nspin; ma++) if(trija[ma][n1] != 0) {
			map=ma+n1-k1; /* ma: M=+I..-I   map: Mp=M-N=I+K1..-I-K1 */
		     for (n=0; n<nspin; n++)
	  	       for (np=0; np<nspin; np++) {
				tr=ZR(hr[ma][n],hi[ma][n],hr[map][np],-hi[map][np]);
				ti=ZI(hr[ma][n],hi[ma][n],hr[map][np],-hi[map][np]);
				pr=ZR(tr,ti,yyr,yyi)*trija[ma][n1];
				pi=ZI(tr,ti,yyr,yyi)*trija[ma][n1];
				if(xfabs(pr)>EPS || xfabs(pi)>EPS) {
				  for (mb=0; mb<nspin; mb++) if(trijb[mb][n2] != 0) {
					mbp=mb+n2-k2;
					tmp=trijb[mb][n2]*PM1(-ma-mb);
					tr=ZR(hr[mb][n],-hi[mb][n],hr[mbp][np],hi[mbp][np]);
					ti=ZI(hr[mb][n],-hi[mb][n],hr[mbp][np],hi[mbp][np]);
		     		ar[n][np]+=ZR(pr,pi,tr,ti)*tmp;
					ai[n][np]+=ZI(pr,pi,tr,ti)*tmp;
                    	}  /* mb */
				}  /* if pr.. */
		       }  /* np n */
		   }  /* ma */
		}  /* if yyr.. */
	}  /* n2 n1 */

} else if(k1==k2) {  /* random source when ntf=0 */
   if(ntf==0) yyr=y2r[0][k2k2+k2]; else yyr=(y2r[0][k2k2+k2]-y2r[1][k2k2+k2])*2/3;
   yyr *= akk[k1/2-1][k2/2-1]*sqrt(PI4/(k2+k2+1));
	/*  Legendre(cos theta) for two coincidence angles */
   if(xfabs(yyr)>EPS) {
/*   start summation */
	for (n1=0; n1<=k1+k1; n1++)              /* -K1..+K1,  K1=K2 and N1=N2 */
		for (ma=0; ma<nspin; ma++) if(trija[ma][n1] != 0) {
			map=ma+n1-k1; /* ma: M=+I..-I   map: Mp=M-N=I+K1..-I-K1 */
		     for (n=0; n<nspin; n++)
	  	       for (np=0; np<nspin; np++) {
				pr=hr[ma][n]*hr[map][np]*trija[ma][n1]*yyr;
				if(xfabs(pr)>EPS)
				  for (mb=0; mb<nspin; mb++) if(trija[mb][n1] != 0) {
					mbp=mb+n1-k1;
					ar[n][np]+=pr*hr[mb][n]*hr[mbp][np]*trija[mb][n1]*
							 PM1(-ma-mb);
                      }  /* mb */
		       }  /* np n */
		   }  /* ma */
   }  /* if yyr... */
}     /* if ntf... */
  }   /* k1, k2 */
/*	store cos and sin term  */
   k=0;
   w[n_][0]=0;
   cosa[n_][0]=0;
   sina[n_][0]=0;
   for (i=0; i<nspin; i++) {
      cosa[n_][0]+=ar[i][i];
         for (j=i+1; j<nspin; j++) {
            w[n_][++k]=ev[j]-ev[i];
            cosa[n_][k]=ar[j][i]+ar[i][j];
            if(ntf>0) sina[n_][k]=ai[j][i]-ai[i][j]; else sina[n_][k]=0;
            if(xfabs(cosa[n_][k])<EPS && xfabs(sina[n_][k])<EPS) k-=1;
         else if(w[n_][k]<0) { /* make positive */
            w[n_][k]=-w[n_][k];
            sina[n_][k]=-sina[n_][k];
         }
      }
   }
/*      sort in energies */
   for (i=0; i<k; i++) {
      n=i;
      for (j=i+1; j<=k; j++) if(w[n_][j] > w[n_][n]) n=j;
      tmp=w[n_][i];
      w[n_][i]=w[n_][n];
      w[n_][n]=tmp;
      tmp=cosa[n_][i];
      cosa[n_][i]=cosa[n_][n];
      cosa[n_][n]=tmp;
      tmp=sina[n_][i];
      sina[n_][i]=sina[n_][n];
      sina[n_][n]=tmp;
	}
/*	sum equal energies */
   n=0;
   for (i=1; i<=k; i++) {
      if(xfabs(w[n_][i]-w[n_][n]) < EPS*w[n_][0]) {
         cosa[n_][n]+=cosa[n_][i];
         sina[n_][n]+=sina[n_][i];
      } else {
         n+=1;
         w[n_][n]=w[n_][i];
         cosa[n_][n]=cosa[n_][i];
         sina[n_][n]=sina[n_][i];
      }
   }
   return(n+1);
} /* end wkk */

/* $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ */

/* ----------------------------------------------------------------
   You should allways supply a help function, which will be called
   when no parameter is specified.
   ---------------------------------------------------------------- */

help()
{
printf("External Fit function for combined interaction:\n");
printf("Parameter:\n");
printf("p[0] = -1 (or 0) for Poly Crystal, +1 for Single Crystal\n");
printf("p[1] = t0       P[2] = spin        p[3] = a0\n");
printf("p[4] = tau      p[5] = A22         p[6] = A24\n");
printf("p[7] = A42      p[8] = A44         p[9] = stop1.stop2\n");
printf("p[10]= w0       p[11]= eta         p[12]= wb\n");
printf("p[13]= beta     p[14]= gamma       p[15]= Euler ampl\n");
printf("p[16]= alpha    p[17]= beta        p[18]= gamma\n");
printf("p[19]= damp     p[20]= Euler frac  p[21]= alpha\n");
printf("p[22]= beta     p[23]= gamma       p[24]= FHWHM prompt\n\n");
printf("the stop detector angles are separated by a '.' !\n");
printf("(Stop1=180, stop2=90 => p[9]=180.090)\n");
printf("MAXSPIN = %d    (change #define and recompile !)\n",MAXSPIN);
return(0);
}

/* ----------------------------------------------------------------
      shared memory allocation
   ---------------------------------------------------------------- */
alloc_shm()
{
#ifdef AMIGA
   shmx = (double *) shmidx;
   shmy = (double *) shmidy;
#endif
#ifdef UNIX
   shmx = (double *) shmat(shmidx, 0,0666);
   shmy = (double *) shmat(shmidy, 0,0666);
   if((shmx < 0) || (shmy < 0)) {
      fprintf(stderr,"could not attach Shared memory, aborting !\n");
      exit(-1);
   }
#endif
   return(0);
}

free_shm()
{
#ifdef UNIX
   shmdt(shmx);
   shmdt(shmy);
#endif
   return(0);
}

main(argc,argv)
int argc;
char **argv;
{
int n;

   if(argc < 3) {help(); exit(0);}

   q = (wkkarea*) malloc(sizeof(wkkarea));
   if (q == NULL) {
      printf(" malloc failed for f_xtal or f_nonco\n");
      exit(1);
   }
   q->nm=-1;

   p = (double *) calloc(MAXPARM,sizeof(double));

   shmidx = atoi(argv[1]);
   shmidy = atoi(argv[2]);
   alloc_shm();
   first = atoi(argv[3]);
   last = atoi(argv[4]);
   max = atoi(argv[5]);
   for(n = 0; n < MAXPARM; n++) {
      sscanf(argv[n + 6],"%lf",&p[n]);
   }
   tchan = shmx[1] - shmx[0];
   xtal_flag = POLY;
   if(p[0] > EPS) xtal_flag = XTAL;
   t0 = p[1];
   spin = p[2];
   a0 = p[3];
   tau = p[4];
   a22 = p[5];
   a24 = p[6];
   a42 = p[7];
   a44 = p[8];
      
   stop2 = (double) modf((double)p[9],&stop1);
   stop2 = 1000.0 * stop2;

   w0 = p[10];
   eta = p[11];
   wb = p[12];
   beta_1 = p[13];
   gamma_1 = p[14];
   euler_amp = p[15];
   alpha_2 = p[16];
   beta_2 = p[17];
   gamma_2 = p[18];
   damp = p[19];
   euler_frac = p[20];
   alpha_3 = p[21];
   beta_3 = p[22];
   gamma_3 = p[23];
   fwhm = p[24];
   
   n_wkk = prewkk();
   for(n = first; n <= last; n++) {
      shmy[n] = (double) fn((double) shmx[n]);
   }
   free_shm();
   free(p);
   free(q);
   exit(0);
}

